/*
 * VARIABLES
 */
PULSE_TIME = 25; // 1/1000 seconds
GRAVITY = 3.5;
RESISTANCE = 0.98; // air resitance, approx (obviously)
BLURMIN = 70; // when velocity is over this then we fake motion blur
FRICTION = 0.95; // this is actually more of a colision penalty
CUSHION = 2; // amount, in pixels, that the edge will 'absorb' the ball
FLOOR = 10; // where the floor is, from the edge
SHADOWDIST = 300; // distance at which shadow disappears
ESCAPE = 200; // if the ball goes faster than this it is thrown
held = false; // the ball has been grabbed by the mouse
    // these are optimistic defaults
XMIN = 0;
YMIN = 0;
XMAX = 1024;
YMAX = 720;
oldy = oldx = 0;
dragy = dragx = 0;
pulseid = null;
hidden = false; // to stop both onload and onshow from calling startPhysics

/*
 * EVENT HANDLERS
 */
function onload() {
    redball = document.getElementById("redball");
    shadow = document.getElementById("shadow");
    reflec = document.getElementById("reflection");
    shadowheight = Math.round(shadow.offsetHeight / 2);
    blur1 = document.getElementById("blur1");
    blur2 = document.getElementById("blur2");
    blur3 = document.getElementById("blur3");
    redball.onmousedown = onGrab;
    setupBoundaries();
    startPhysics();    
}
function onhide() {
    // the ball will freeze
    stopPhysics();
    hidden = true;
}
function onshow() {
    // the ball should drop from where it was frozen
    if (hidden) {
        startPhysics();
        hidden = false;
    }
}
function onGrab(e) {
    held = true;
    grabX = e.pageX - redball.offsetLeft;
    grabY = e.pageY - redball.offsetTop;
    dragx = e.pageX - grabX;
    dragy = e.pageY - grabY;
    oldx = dragx;
    oldy = dragy;
}
function onThrow() {
    held = false;
    yvel *= 0.7;
    xvel *= 0.7;
}
function onMove(e) {
    if (held) {
        dragx = e.pageX - grabX;
        dragy = e.pageY - grabY;
        //redball.style.left = e.pageX-grabX +"px";
        //redball.style.top = e.pageY-grabY +"px";
    }
}

/*
 * SETUP EVENT CAPTURE
 */
    //setup mouse move capture
window.document.onmousemove = onMove;
window.document.onmouseup = onThrow;
window.onload = onload;

//setup widget status capture
if (window.widget)
{
    widget.onshow = onshow;
    widget.onhide = onhide;
}

/*
 * PHYSICS
 */
function setupBoundaries() {
    XMIN = window.screenLeft * -1 - CUSHION;
    XMIN = Math.max(XMIN, window.screenLeft);
    XMAX = XMIN + window.screen.width - redball.offsetWidth + CUSHION * 2;
    XMAX = Math.min(XMAX, window.outerWidth - redball.offsetWidth);
    YMIN = window.screenTop * -1 - CUSHION;
    YMAX = YMIN + window.screen.height - redball.offsetHeight - FLOOR;
    YMAX = Math.min(YMAX, window.outerHeight - redball.offsetHeight - FLOOR);
}
function startPhysics() {
    xvel = 0.0;
    yvel = 0.0;
    pulseid = setInterval("pulse()", PULSE_TIME);
}
function stopPhysics() {
    clearInterval(pulseid);
}
stopdate = new Date();
function pulse() {
    var startdate = new Date();
    //alert(startdate.getTime() - stopdate.getTime());
    if (held) {
        //newx = redball.offsetLeft;
        //newy = redball.offsetTop;
        xvelold = xvel;
        yvelold = yvel;
        yvel = dragy - oldy;
        xvel = dragx - oldx;
        if (Math.sqrt(xvel*xvel + yvel*yvel) > ESCAPE) {
            held = false;
        }
        oldx = dragx;
        oldy = dragy;
        drawBall(Math.round(dragx), Math.round(dragy), xvel, yvel);
    } else {
        // Y
        yvel = yvel + GRAVITY;
        newy = redball.offsetTop + yvel;
        if (newy > YMAX) {
            yvel = (yvel * -1 + GRAVITY) * FRICTION; // bounce the other way
            if (Math.abs(yvel) > 50) {
                newy = 2*YMAX - newy;
            } else {
                newy = YMAX;
            }
        } else if (newy < YMIN) {
            yvel = yvel * -1 * FRICTION;
            newy = YMIN + (YMIN - newy);
        }
        // X
        xvel *= RESISTANCE;
        //if (xvel > -0.5 && xvel < 0.5)
          //  xvel = 0;
        newx = redball.offsetLeft + xvel;
        if (newx > XMAX) {
            xvel *= -1;
            xvel *= FRICTION;
            newx = 2*XMAX - newx;
            
        } else if (newx < XMIN) {
            xvel *= -1;
            xvel *= FRICTION;
            newx = XMIN + (XMIN - newx);
            
        }
        // if after all that they still end up off screen - say a crazy velocity - then get them back
        if (newx > XMAX) {
            newx = XMAX;
        }
        if (newx < XMIN) {
            newx = XMIN; //some velocities will take it off screen both ways
        }
        if (newy == YMAX) {
            xvel *= FRICTION; // gliding along the floor 
        }
        drawBall(Math.round(newx), Math.round(newy), xvel, yvel); //handles motion blur, shadow
        //redball.style.top = Math.round(newy)+"px";
        //redball.style.left = Math.round(newx)+"px";
    }
    widget.setCloseBoxOffset(redball.offsetLeft+20, redball.offsetTop+20);
    stopdate = new Date();
}
function drawBall(x, y, xvel, yvel) {
    if (Math.sqrt(xvel*xvel + yvel*yvel) > BLURMIN) {
        blur1.style.display="";
        blur2.style.display="visible";
        blur3.style.display="visible";
        
        blur1.style.left = x - (xvel * 0.6) +"px";
        blur2.style.left = x - (xvel * 0.4) +"px";
        blur3.style.left = x - (xvel * 0.2) +"px";
        
        blur1.style.top = y - (yvel * 0.6) +"px";
        blur2.style.top = y - (yvel * 0.4) +"px";
        blur3.style.top = y - (yvel * 0.2) +"px";
        
        blur1.style.opacity = 0.1;
        blur2.style.opacity = 0.18;
        blur3.style.opacity = 0.26;
        redball.style.opacity = 0.6;
    } else {
        blur1.style.display="none";
        blur2.style.display="none";
        blur3.style.display="none";
        redball.style.opacity = 1.0;
    }
    reflec.style.top = (YMAX*2 - y + reflec.offsetHeight - 3)+"px"; //2 is for pseudo perpective/overlap
    reflec.style.left = x+"px";
    shadow.style.top = (YMAX + shadowheight) +"px";
    shadow.style.left = x+"px";
    shadow.style.opacity = 1.0 - Math.min(1, (YMAX - y) / SHADOWDIST);
    redball.style.left = x+"px";
    redball.style.top = y+"px";
}